library mover;

uses
  SysUtils,messages,windows;

{R English.RES}
{$R Francais.res}
{$E wcx}
const       {Error codes returned to calling application}
  E_END_ARCHIVE=     10;       {No more files in archive}
  E_NO_MEMORY=       11;       {Not enough memory}
  E_BAD_DATA=        12;       {Data is bad}
  E_BAD_ARCHIVE=     13;       {CRC error in archive data}
  E_UNKNOWN_FORMAT=  14;       {Archive format unknown}
  E_EOPEN=           15;       {Cannot open existing file}
  E_ECREATE=         16;       {Cannot create file}
  E_ECLOSE=          17;       {Error closing file}
  E_EREAD=           18;       {Error reading from file}
  E_EWRITE=          19;       {Error writing to file}
  E_SMALL_BUF=       20;       {Buffer too small}
  E_EABORTED=        21;       {Function aborted by user}
  E_NO_FILES=        22;       {No files found}
  E_TOO_MANY_FILES=  23;       {Too many files to pack}
  E_NOT_SUPPORTED=   24;       {Function not supported}

  {Returned by GetPackCaps}
  PK_CAPS_NEW=          1;    {Can create new archives}
  PK_CAPS_MODIFY=       2;    {Can modify exisiting archives}
  PK_CAPS_MULTIPLE=     4;    {Archive can contain multiple files}
  PK_CAPS_DELETE=       8;    {Can delete files}
  PK_CAPS_OPTIONS=     16;    {Supports the options dialogbox}
  PK_CAPS_MEMPACK=     32;    {Supports packing in memory}
  PK_CAPS_BY_CONTENT=  64;    {Detect archive type by content}
  PK_CAPS_SEARCHTEXT= 128;    {Allow searching for text in archives created with this plugin}
  PK_CAPS_HIDE=       256;    {Don't show as plugin, needs to open with Ctrl+PgDn}


type
  pbyte=^byte;
  {Definition of callback functions called by the DLL}
  {Ask to swap disk for multi-volume archive}
  PChangeVolProc=^TChangeVolProc;
  TChangeVolProc=function(ArcName:pchar;Mode:longint):longint; stdcall;
  {Notify that data is processed - used for progress dialog}
  PProcessDataProc=^TProcessDataProc;
  TProcessDataProc=function(Addr:pbyte;Size:longint):longint; stdcall;

type
  THeaderData=packed record
    ArcName:array [0..259] of char;
    FileName:array [0..259] of char;
    Flags,
    PackSize,
    UnpSize,
    HostOS,
    FileCRC,
    FileTime,
    UnpVer,
    Method,
    FileAttr:longint;
    CmtBuf:pchar;
    CmtBufSize,
    CmtSize,
    CmtState:longint;
  end;

  tOpenArchiveData=packed record
    ArcName:pchar;
    OpenMode,
    OpenResult:longint;
    CmtBuf:pchar;
    CmtBufSize,
    CmtSize,
    CmtState:longint;
  end;

const maxarchives=10;
type ArchiveRec=record
       ArchiveName:array[0..259] of char;
       SrcDir,LastCurDir:array[0..259] of char;
       CurrentProcessedFile:THeaderData;
       ArchiveHandle:textfile;
       ArchiveMode:longint;
       ArchiveActive:boolean;
       ChangeVolProc:TChangeVolProc;
       ProcessDataProc:TProcessDataProc;
     end;

{Keep a list of currently open archives (up to a maximum of maxarchives)}
var ArchiveList:array[1..maxarchives] of ArchiveRec;
    NbOnly:boolean=false;
    DatStamp:boolean=false;
    Mo_file:boolean=false;
    Day_File:boolean=false;
    File_type:boolean=false;

function OpenArchive(var ArchiveData:tOpenArchiveData):thandle; stdcall;
begin
  result:=0;
  ArchiveData.OpenResult:=E_UNKNOWN_FORMAT;
  exit
end;

function CloseArchive(hArcData:thandle):longint; stdcall;
begin
  Result:=0;
end;

type
  TDateTime1 = record
    Year, Month, Day, Hour, Min, Sec: Word;
  end;

function ReadHeader(hArcData:thandle;var HeaderData:THeaderData):longint; stdcall;
begin
  Result:=E_BAD_ARCHIVE;
end;

function ProcessFile(hArcData:thandle;Operation:longint;DestPath,DestName:pchar):longint; stdcall;
begin
  Result:=E_BAD_ARCHIVE;
end;

var PackerChangeVolProc:TChangeVolProc;
    PackerProcessDataProc:TProcessDataProc;

function SetChangeVolProc(hArcData:THandle;ChangeVolProc1:TChangeVolProc):longint; stdcall;
begin
  PackerChangeVolProc:=ChangeVolProc1;
end;

function SetProcessDataProc(hArcData:THandle;ProcessDataProc1:TProcessDataProc):longint; stdcall;
begin
  PackerProcessDataProc:=ProcessDataProc1;
end;

{==========================================================================}

type  { TMessage windows message record }
  PMessage = ^TMessage;
  TMessage = record
    Receiver: HWnd;
    Message: Word;
    case Integer of
      0: (
        WParam: Word;
        LParam: Longint;
        Result: Longint);
      1: (
        WParamLo: Byte;
        WParamHi: Byte;
        LParamLo: Word;
        LParamHi: Word;
        ResultLo: Word;
        ResultHi: Word);
  end;

const
  id_Input  = 8;
  id_NbOnly=3;
  id_DatStamp=4;
  id_Day_File=5;
  id_Mo_file=6;
  id_Type=9;


var Caption: PChar;
    Prompt: PChar;
    Buffer: PChar;
    BufferSize: Word;
    br:hbrush;
    ininame:array[0..259] of char;

procedure TMyInputDialog_Init(ACaption, APrompt, ABuffer: PChar; ABufferSize: Word);
begin
  Caption := ACaption;
  Prompt := APrompt;
  Buffer := ABuffer;
  BufferSize := ABufferSize;
end;

function TMyInputDialog_CanClose(var msg:tmessage):Boolean;
begin
  GetDlgItemText(msg.receiver, id_Input, Buffer, BufferSize);
  TMyInputDialog_CanClose := True;
end;

procedure TMyInputdialog_setupwindow(var msg:tmessage);
var r,pr:trect;
    centerx,centery,hwindow,i:integer;
    buf1:array[0..15] of char;
begin
  hwindow:=msg.receiver;
  br:=createsolidbrush(getsyscolor(color_btnface));
  SetWindowText(HWindow, Caption);
  for i:=0 to 19 do begin
    str(i,buf1);
  end;
  SendDlgItemMessage(HWindow, id_Input, em_LimitText, BufferSize - 1, 0);

  Windows.setfocus(GetDlgItem(hwindow,id_Input));

  getwindowrect(hwindow,r);
  getwindowrect(getparent(hwindow),pr);
  centerx:=pr.left+((pr.right-pr.left)-(r.right-r.left)) div 2;
  centery:=pr.top+((pr.bottom-pr.top)-(r.bottom-r.top)) div 2;
  movewindow(hwindow,centerx,centery,r.right-r.left,r.bottom-r.top,true);
end;

procedure TMyInputdialog_Confirmer(var msg:tmessage);
var hwindow:integer;
    newline:array[0..259] of char;
begin
  hwindow:=msg.receiver;
  GetDlgItemText(hwindow,id_Input,newline,sizeof(newline)-1);

  NbOnly:=IsDlgButtonChecked(HWindow,id_nbonly)=BST_CHECKED;
  DatStamp:=IsDlgButtonChecked(HWindow,id_DatStamp)=BST_CHECKED;;
  Day_File:=IsDlgButtonChecked(HWindow,id_Day_File)=BST_CHECKED;
  Mo_file:=IsDlgButtonChecked(HWindow,id_Mo_file)=BST_CHECKED;
  File_type:=IsDlgButtonChecked(HWindow,id_Type)=BST_CHECKED;
  deleteobject(br);
end;

procedure TMyInputdialog_Annuler(var msg:tmessage);
begin
  deleteobject(br);
end;

procedure TMyInputdialog_wmctlcolor(var msg:tmessage);
begin
  if msg.lparamhi<>ctlcolor_edit then begin
    msg.result:=br;
    SetBkColor(msg.wparam,getsyscolor(color_btnface));
    SetTextColor(msg.wparam,getsyscolor(color_btntext));
  end else begin
    msg.result:=getstockobject(white_brush);
    SetBkColor(msg.wparam,rgb(255,255,255));
    SetTextColor(msg.wparam,0);
  end;
end;

function MyInputDlgProc(Dialog: HWnd; Message, WParam: Word;
  LParam: Longint): integer; export; {$ifdef win32} stdcall; {$endif}
var msg:tmessage;
begin
  MyInputDlgProc:=1;
  Msg.Receiver:=dialog;
  Msg.Message:=Message;
  Msg.wparam:=wparam;
  Msg.lparam:=lparam;
  Msg.Result:=1;
  case Message of
    wm_InitDialog:begin
      TMyInputDialog_SetupWindow(msg);
      exit
    end;
    wm_ctlcolor:begin
      TMyInputDialog_wmctlcolor(msg);
      MyInputDlgProc:=msg.result;
      exit
    end;
    wm_command:case msg.wparam of
      id_ok:begin
        if not TMyInputdialog_CanClose(msg) then exit;
        TMyInputDialog_Confirmer(msg);
        EndDialog(Dialog, id_ok);
        Exit;
      end;
      id_cancel:begin
        if not TMyInputdialog_CanClose(msg) then exit;
        TMyInputDialog_Annuler(msg);
        EndDialog(Dialog, id_cancel);
        Exit;
      end;
    end;
  end;
  MyInputDlgProc:=0;
end;
function IsNumber(s: string): Boolean;
var 
  i: Integer;
begin
  Result := False;
  for i := 1 to Length(s) do
    case s[i] of
      '0'..'9':;
      else 
        Exit;
    end;
  Result := True;
end;

function PackFiles(PackedFile,SubPath,SrcPath,AddList:pchar;Flags:integer):integer; stdcall;
var s:tsearchrec;
   p, pnextname:pchar;
    template:array[0..9] of char;
    rc:integer;
    RunOk:boolean;
    s1,s_month,s_day,S2:string;
    i,j,k,data:integer;
    wYear,wMonth,wDay:word;
    annee:boolean;
begin
  annee:=false;
  template[0]:=#0;
  GetModuleFileName(hinstance,ininame,sizeof(ininame)-1);
  TMyInputDialog_Init('Dplac X fichiers','',template,sizeof(template)-1);
  rc:=dialogbox(hinstance,'MyInputDialog',GetActiveWindow,@MyInputDlgProc);
  if rc<>id_ok then begin
    result:=E_EABORTED;
    exit
  end;
  pnextname:=AddList;
  RunOK:=true;
  if not (NbOnly or DatStamp or File_type) then begin
    MessageBox(GetActiveWindow,'Choissisez au moins une mthode','Erreur',MB_ICONERROR);
  end;
  if (NbOnly and DatStamp) or (NbOnly and File_type) or (DatStamp and File_type) or (DatStamp and NbOnly and File_type) then begin
    MessageBox(GetActiveWindow,'Vous ne pouvez slectionner qu''une seule mthode','Erreur',MB_ICONERROR);
  end else begin
    if NbOnly then begin
      i:=1;
      j:=1;
      if isnumber(template) then begin
        k:=StrToInt(template);
        data:=k;
        while (pnextname[0]<>#0) and RunOK do begin
          p:=strend(pnextname)-1;
          if p[0]<>'\' then begin   {Only files, no dirs!}
            if @PackerProcessDataProc<>nil then if PackerProcessDataProc(pByte(pnextname),s.size)=0 then RunOK:=false;
            SetCurrentDir(SrcPath);
            if i>k then begin //On a dpass lenombre de fichiers par rpertoire
               k:=k+data      ; //On dcale le compteur
               inc(j); //On incrmente le nom du rpertoire cible
            end;
            if j<=9 then s1:='0'+inttostr(j) else s1:=inttostr(j);//Nom du rpertoire cible
            CreateDir(SrcPath+s1); //Cration du rpertoire courant

            S2:=ExtractFilePath(pnextname)+s1+'\'+ExtractFileName(pnextname); //Nouveau nom pour le fichier
            RenameFile(pnextname,S2);
            inc(i); //On augmente le compteur global de fichier
          end;
          pnextname:=strend(pnextname)+1;
        end;
      end else MessageBox(GetActiveWindow,'Il faut rentrer un Nombre !','Attention',MB_ICONWARNING);
    end;
    if DatStamp then begin
      if ((Mo_file and Day_file)) or (not Mo_file and not Day_file) then begin
          MessageBox(GetActiveWindow,'Problme dans le choix Jour/Mois pour le tri, le module va utiliser l''anne seulement','Attention',MB_ICONWARNING);
          annee:=true;
      end;
      while (pnextname[0]<>#0) and RunOK do begin
        p:=strend(pnextname)-1;
        if p[0]<>'\' then begin   {Only files, no dirs!}
        if @PackerProcessDataProc<>nil then if PackerProcessDataProc(pByte(pnextname),s.size)=0 then RunOK:=false;
        SetCurrentDir(SrcPath);
        data:=FileAge(pnextname);
        DecodeDate(FileDateToDateTime(data),wYear,wMonth,wDay);
        if annee then begin
          CreateDir(SrcPath+inttostr(wYear));
          S2:=ExtractFilePath(pnextname)+inttostr(wYear)+'\'+ExtractFileName(pnextname);
        end
        else begin
          if wMonth<10 then s_month:='0'+inttostr(wMonth)
          else s_month:=inttostr(wMonth);
          if wDay<10 then s_Day:='0'+inttostr(wDay)
          else s_Day:=inttostr(wDay);
          if Mo_file then begin
              CreateDir(SrcPath+inttostr(wYear)+s_month);
              S2:=ExtractFilePath(pnextname)+inttostr(wYear)+s_month+'\'+ExtractFileName(pnextname);
          end;
          if Day_File then begin
            CreateDir(SrcPath+inttostr(wYear)+s_Month+s_Day);
            S2:=ExtractFilePath(pnextname)+inttostr(wYear)+s_Month+s_Day+'\'+ExtractFileName(pnextname);
          end;
        end;
        RenameFile(pnextname,S2);
      end;
      pnextname:=strend(pnextname)+1;
    end;
  end;
  if File_Type then begin
     while (pnextname[0]<>#0) and RunOK do begin
        p:=strend(pnextname)-1;
        if p[0]<>'\' then begin   {Only files, no dirs!}
        if @PackerProcessDataProc<>nil then if PackerProcessDataProc(pByte(pnextname),s.size)=0 then RunOK:=false;
        SetCurrentDir(SrcPath);
        s_day:=ExtractFileExt(pnextname);
        if s_day<>'' then begin
        s_day:=StringReplace(s_day,'.', '',[rfReplaceAll, rfIgnoreCase]);
        CreateDir(SrcPath+s_day);
        S2:=ExtractFilePath(pnextname)+s_day+'\'+ExtractFileName(pnextname);
        RenameFile(pnextname,S2);
        end;
      end;
      pnextname:=strend(pnextname)+1;
    end;
  end;
    end;
  if RunOK then result:=0
  else result:=E_EABORTED;
end;

function DeleteFiles(PackedFile,DeleteList:pchar):integer; stdcall;
begin
  result:=E_NOT_SUPPORTED;
end;

function GetPackerCaps:integer; stdcall;
begin
  result:=PK_CAPS_NEW or PK_CAPS_MULTIPLE or PK_CAPS_MODIFY or PK_CAPS_OPTIONS or PK_CAPS_HIDE;
end;

procedure ConfigurePacker(ParentHandle,DllInstance:thandle); stdcall;
begin
  MessageBox(ParentHandle,'Fait trs rapidement pour les besoins d''un utilisateur FR...',
    'A propos de Mover',mb_iconinformation);
end;

exports
  OpenArchive,
  CloseArchive,
  ReadHeader,
  ProcessFile,
  SetChangeVolProc,
  SetProcessDataProc,
  PackFiles,
  DeleteFiles,
  GetPackerCaps,
  ConfigurePacker;

begin
  FillChar(ArchiveList,sizeof(ArchiveList),#0);
end.
